use std::{mem, io, thread};
use std::pin::Pin;
use std::task::{Context, Poll};

use termion::event::Key;
use termion::input::TermRead;

use futures::Stream;
use futures::ready;
use futures::channel::mpsc;
use futures::SinkExt;

pub struct Inputs {
    rx: mpsc::Receiver<Key>,
    hndl: thread::JoinHandle<()>,
    cmd_line: String,
}

impl Inputs {
    pub fn new() -> Self {
        let (mut tx, rx) = mpsc::channel(64);

        let hndl = thread::spawn(move || {
            let stdin = io::stdin();
            let mut keys = stdin.keys();
            for key in keys {
                let key = key.unwrap();
                smol::block_on(tx.send(key));
            }
        });

        Self {
            rx: rx,
            hndl: hndl,
            cmd_line: String::new(),
        }
    }

    pub fn get_line(&self) -> &str {
        self.cmd_line.as_ref()
    }

    pub fn handle_key(&mut self, key: Key) -> Option<String> {
        match key {
            Key::Char('\n') => Some(mem::replace(&mut self.cmd_line, String::new())),
            Key::Char(c) => {
                self.cmd_line.push(c);
                None
            },
            Key::Backspace => {
                self.cmd_line.pop();
                None
            },
            _ => None
        }
    }
}

impl Stream for Inputs {
    type Item = String;

    fn poll_next(mut self: Pin<&mut Self>, cx: &mut Context) -> Poll<Option<Self::Item>> {
        if let Some(key) = ready!(Pin::new(&mut self.rx).poll_next(cx)) {
            if let Some(line) = self.handle_key(key) {
                Poll::Ready(Some(line))
            } else {
                Poll::Pending
            }
        } else {
            Poll::Pending
        }
    }
}
